# -*- coding: utf-8 -*-
# Generated by Django 1.11.23 on 2019-08-29 20:03
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0028_agegroups'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subjectpayment',
            name='related_payment',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='related_payments', to='leprikon.SubjectPayment', verbose_name='related payment'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2019-09-20 21:11
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import leprikon.models.fields
import leprikon.models.subjects


def set_subject_registration_type(apps, schema_editor):
    Subject = apps.get_model('leprikon', 'Subject')
    Subject.objects.filter(max_group_members_count__gt=0).update(
        registration_type='G',
        min_participants_count=0,
        max_participants_count=0,
    )


def create_subject_registration_groups(apps, schema_editor):
    SubjectRegistration = apps.get_model('leprikon', 'SubjectRegistration')
    SubjectRegistrationGroup = apps.get_model('leprikon', 'SubjectRegistrationGroup')
    SubjectRegistrationGroup.objects.bulk_create(
        SubjectRegistrationGroup(
            registration=registration,
            target_group_id=registration.target_group_id,
            name=registration.group_name,
            first_name=registration.group_leader_first_name,
            last_name=registration.group_leader_last_name,
            street=registration.group_leader_street,
            city=registration.group_leader_city,
            postal_code=registration.group_leader_postal_code,
            phone=registration.group_leader_phone,
            email=registration.group_leader_email,
            school_id=registration.group_school_id,
            school_other=registration.group_school_other,
            school_class=registration.group_school_class,
        )
        for registration in SubjectRegistration.objects.filter(subject__registration_type='G')
    )


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0033_target_groups'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subject',
            name='age_groups',
            field=models.ManyToManyField(related_name='subjects', to='leprikon.AgeGroup', verbose_name='age groups'),
        ),
        migrations.AlterField(
            model_name='subject',
            name='target_groups',
            field=models.ManyToManyField(related_name='subjects', to='leprikon.TargetGroup', verbose_name='target groups'),
        ),
        migrations.AlterField(
            model_name='subject',
            name='max_group_members_count',
            field=models.PositiveIntegerField(help_text='Group member details only include name and note.', verbose_name='maximal group members count per registration'),
        ),
        migrations.AlterField(
            model_name='subject',
            name='min_group_members_count',
            field=models.PositiveIntegerField(help_text='Group member details only include name and note.', verbose_name='minimal group members count per registration'),
        ),
        migrations.AddField(
            model_name='subject',
            name='registration_type',
            field=models.CharField(choices=[('P', 'participants'), ('G', 'groups')], default='P', max_length=1, verbose_name='registration type'),
            preserve_default=False,
        ),
        migrations.RunPython(set_subject_registration_type),
        migrations.CreateModel(
            name='SubjectRegistrationGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('registration', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='group', to='leprikon.SubjectRegistration', verbose_name='registration')),
                ('target_group', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='leprikon.TargetGroup', verbose_name='target group')),
                ('name', models.CharField(blank=True, null=True, max_length=150, verbose_name='group name')),
                ('first_name', models.CharField(max_length=30, verbose_name='first name')),
                ('last_name', models.CharField(max_length=30, verbose_name='last name')),
                ('street', models.CharField(max_length=150, verbose_name='street')),
                ('city', models.CharField(max_length=150, verbose_name='city')),
                ('postal_code', leprikon.models.fields.PostalCodeField(verbose_name='postal code')),
                ('phone', models.CharField(max_length=30, verbose_name='phone')),
                ('email', leprikon.models.fields.EmailField(max_length=254, verbose_name='email address')),
                ('school', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='leprikon.School', verbose_name='school')),
                ('school_other', models.CharField(blank=True, default='', max_length=150, verbose_name='other school')),
                ('school_class', models.CharField(blank=True, default='', max_length=30, verbose_name='class')),
                ('answers', models.TextField(blank=True, default='{}', editable=False, verbose_name='additional answers')),
            ],
            options={
                'verbose_name': 'registered group',
                'verbose_name_plural': 'registered groups',
            },
            bases=(leprikon.models.subjects.SchoolMixin, leprikon.models.subjects.PersonMixin, models.Model),
        ),
        migrations.RunPython(create_subject_registration_groups),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_age_group',
            new_name='age_group',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_birth_num',
            new_name='birth_num',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_citizenship',
            new_name='citizenship',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_city',
            new_name='city',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_email',
            new_name='email',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_first_name',
            new_name='first_name',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_gender',
            new_name='gender',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_health',
            new_name='health',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_last_name',
            new_name='last_name',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_phone',
            new_name='phone',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_postal_code',
            new_name='postal_code',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_school',
            new_name='school',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_school_class',
            new_name='school_class',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_school_other',
            new_name='school_other',
        ),
        migrations.RenameField(
            model_name='subjectregistrationparticipant',
            old_name='participant_street',
            new_name='street',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='group_leader_city',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='group_leader_email',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='group_leader_first_name',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='group_leader_last_name',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='group_leader_phone',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='group_leader_postal_code',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='group_leader_street',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='group_name',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='group_school',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='group_school_class',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='group_school_other',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='target_group',
        ),
    ]



from django.shortcuts import get_object_or_404, redirect

from ..models.registrationlink import RegistrationLink
from ..models.subjects import Subject
from .subjects import (
    SubjectListBaseView, SubjectRegistrationFormBaseView, SubjectTypeMixin,
)


class RegistrationLinkMixin(SubjectTypeMixin):
    def dispatch(self, request, registration_link, *args, **kwargs):
        self.registration_link = get_object_or_404(RegistrationLink, slug=registration_link)
        self.subject_type = self.registration_link.subject_type
        self.model = self._models[self.subject_type.subject_type]
        return super(SubjectTypeMixin, self).dispatch(request, *args, **kwargs)

    def get_placeholder(self):
        return super().get_placeholder() + ':' + self.subject_type.slug

    def get_queryset(self):
        return super().get_queryset().filter(registration_links=self.registration_link)

    def get_template_names(self):
        return [
            'leprikon/{}{}.html'.format(self.subject_type.slug, self.template_name_suffix),
            'leprikon/{}{}.html'.format(self.subject_type.subject_type, self.template_name_suffix),
            'leprikon/subject{}.html'.format(self.template_name_suffix),
        ]


class RegistrationLinkView(RegistrationLinkMixin, SubjectListBaseView):
    pass


class SubjectMixin:
    def dispatch(self, request, pk, **kwargs):
        lookup_kwargs = {'subject_type_id': self.registration_link.subject_type_id, 'id': int(pk)}
        if not self.request.user.is_staff:
            lookup_kwargs['public'] = True
        self.subject = get_object_or_404(Subject, **lookup_kwargs)
        if not self.registration_link.registration_allowed:
            return redirect(self.subject)
        self.request.school_year = self.subject.school_year
        return super().dispatch(request, **kwargs)


class RegistrationLinkFormView(RegistrationLinkMixin, SubjectMixin, SubjectRegistrationFormBaseView):
    def form_valid(self, form):
        response = super().form_valid(form)
        self.object.registration_link = self.registration_link
        self.object.save()
        return response



import csv

from django.contrib import admin
from django.test import RequestFactory, TestCase
from django.utils.encoding import force_text


class SimpleTest(TestCase):
    def test_exports(self):
        request = RequestFactory().post('/')  # export doesn't care about request
        for a in (a for a in admin.site._registry.values() if hasattr(a, 'export_as_csv')):
            content = a.export_as_csv(request, a.get_queryset(request)).content
            self.assertLess(0, len(content))
            content = content.decode()
            reader = csv.reader(content.strip().split('\n'))
            num_columns = len(a.get_list_export(request))
            self.assertTrue(all(len(row) == num_columns for row in reader))



import os
import sys


def main():
    os.environ.setdefault(
        "DJANGO_SETTINGS_MODULE",
        "{}.settings".format(os.environ.get('SITE_MODULE', 'leprikon.site')),
    )
    from django.core.management import execute_from_command_line
    execute_from_command_line(sys.argv)


if __name__ == "__main__":
    main()



from collections import namedtuple

from cms.models import CMSPlugin
from cms.models.fields import PageField
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.encoding import smart_text
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from djangocms_text_ckeditor.fields import HTMLField
from filer.fields.image import FilerImageField

from ..conf import settings
from ..forms.leaders import LeaderFilterForm
from .agegroup import AgeGroup
from .citizenship import Citizenship
from .fields import BirthNumberField, EmailField, PostalCodeField
from .school import School
from .schoolyear import SchoolYear


class Leader(models.Model):
    user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.PROTECT,
                                related_name='leprikon_leader', verbose_name=_('user'))
    description = HTMLField(_('description'), blank=True, default='')
    photo = FilerImageField(verbose_name=_('photo'), blank=True, null=True, related_name='+', on_delete=models.SET_NULL)
    page = PageField(verbose_name=_('page'), blank=True, null=True, related_name='+', on_delete=models.SET_NULL)
    school_years = models.ManyToManyField(SchoolYear, related_name='leaders', verbose_name=_('school years'))

    class Meta:
        app_label = 'leprikon'
        ordering = ('user__last_name', 'user__first_name')
        verbose_name = _('leader')
        verbose_name_plural = _('leaders')

    def __str__(self):
        return self.full_name or self.user.username

    @cached_property
    def first_name(self):
        return self.user.first_name

    @cached_property
    def last_name(self):
        return self.user.last_name

    @cached_property
    def full_name(self):
        return '{} {}'.format(self.first_name, self.last_name).strip()

    @cached_property
    def all_contacts(self):
        return list(self.contacts.all())

    @cached_property
    def all_public_contacts(self):
        return list(self.contacts.filter(public=True))

    @cached_property
    def all_school_years(self):
        return list(self.school_years.all())

    def get_alternate_leader_entries(self, school_year):
        from .journals import JournalLeaderEntry
        return JournalLeaderEntry.objects.filter(
            timesheet__leader=self,
            journal_entry__subject__school_year=school_year,
        ).exclude(journal_entry__subject__in=self.subjects.all())

    SubjectsGroup = namedtuple('SubjectsGroup', ('subject_type', 'subjects'))

    def get_subjects_by_types(self):
        from .subjects import SubjectType
        return (
            self.SubjectsGroup(
                subject_type=subject_type,
                subjects=subject_type.subjects.filter(leaders=self)
            )
            for subject_type in SubjectType.objects.all()
        )


class Contact(models.Model):
    leader = models.ForeignKey(Leader, on_delete=models.CASCADE, related_name='contacts', verbose_name=_('leader'))
    contact_type = models.CharField(_('contact type'), max_length=30,
                                    choices=settings.LEPRIKON_CONTACT_TYPES)
    contact = models.CharField(_('contact'), max_length=250)
    order = models.IntegerField(_('order'), blank=True, default=0)
    public = models.BooleanField(_('public'), default=False)

    CONTACT_TYPES = dict(settings.LEPRIKON_CONTACT_TYPES)

    class Meta:
        app_label = 'leprikon'
        ordering = ('order',)
        verbose_name = _('contact')
        verbose_name_plural = _('contacts')

    def __str__(self):
        return '{}, {}: {}'.format(self.leader.full_name, self.contact_type_name, self.contact)

    @cached_property
    def contact_type_name(self):
        return self.CONTACT_TYPES[self.contact_type]


class Parent(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE,
                             related_name='leprikon_parents', verbose_name=_('user'))
    first_name = models.CharField(_('first name'), max_length=30)
    last_name = models.CharField(_('last name'), max_length=30)
    street = models.CharField(_('street'), max_length=150)
    city = models.CharField(_('city'), max_length=150)
    postal_code = PostalCodeField(_('postal code'))
    email = EmailField(_('email address'), blank=True, default='')
    phone = models.CharField(_('phone'), max_length=30)

    class Meta:
        app_label = 'leprikon'
        verbose_name = _('parent')
        verbose_name_plural = _('parents')

    def __str__(self):
        return self.full_name

    @cached_property
    def address(self):
        return '{}, {}, {}'.format(self.street, self.city, self.postal_code)

    @cached_property
    def contact(self):
        if self.email and self.phone:
            return '{}, {}'.format(self.phone, self.email)
        else:
            return self.email or self.phone or ''

    @cached_property
    def full_name(self):
        return '{} {}'.format(self.first_name, self.last_name)


class Participant(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE,
                             related_name='leprikon_participants', verbose_name=_('user'))
    age_group = models.ForeignKey(AgeGroup, on_delete=models.PROTECT,
                                  related_name='+', verbose_name=_('age group'))
    first_name = models.CharField(_('first name'), max_length=30)
    last_name = models.CharField(_('last name'), max_length=30)
    birth_num = BirthNumberField(_('birth number'))
    street = models.CharField(_('street'), max_length=150)
    city = models.CharField(_('city'), max_length=150)
    postal_code = PostalCodeField(_('postal code'))
    email = EmailField(_('email address'), blank=True, default='')
    phone = models.CharField(_('phone'), max_length=30, blank=True, default='')
    citizenship = models.ForeignKey(Citizenship, on_delete=models.PROTECT,
                                    related_name='+', verbose_name=_('citizenship'))
    school = models.ForeignKey(School, blank=True, null=True, on_delete=models.PROTECT,
                               related_name='participants', verbose_name=_('school'))
    school_other = models.CharField(_('other school'), max_length=150, blank=True, default='')
    school_class = models.CharField(_('class'), max_length=30, blank=True, default='')
    health = models.TextField(_('health'), blank=True, default='')
    MALE = 'm'
    FEMALE = 'f'
    gender = models.CharField(_('gender'), max_length=1, editable=False,
                              choices=((MALE, _('male')), (FEMALE, _('female'))))

    class Meta:
        app_label = 'leprikon'
        verbose_name = _('participant')
        verbose_name_plural = _('participants')
        unique_together = (('user', 'birth_num'),)

    def __str__(self):
        return _('{first_name} {last_name} ({birth_num})').format(
            first_name=self.first_name,
            last_name=self.last_name,
            birth_num=self.birth_num,
        )

    def validate_unique(self, exclude=None):
        try:
            # perform the all unique checks, do not exclude anything
            super(Participant, self).validate_unique(None)
        except ValidationError:
            # The only unique constraint is on birth_num and user.
            # Let's use nice birth_num related message instead of the default one.
            raise ValidationError(
                message={'birth_num': _('You have already entered participant with this birth number')},
            )

    def save(self, *args, **kwargs):
        self.gender = self.birth_num[2:4] > '50' and self.FEMALE or self.MALE
        super(Participant, self).save(*args, **kwargs)

    @cached_property
    def full_name(self):
        return '{} {}'.format(self.first_name, self.last_name)

    @cached_property
    def address(self):
        return '{}, {}, {}'.format(self.street, self.city, self.postal_code)

    @cached_property
    def contact(self):
        if self.email and self.phone:
            return '{}, {}'.format(self.phone, self.email)
        else:
            return self.email or self.phone or ''

    @cached_property
    def school_name(self):
        return self.school and smart_text(self.school) or self.school_other

    @cached_property
    def school_and_class(self):
        if self.school_name and self.school_class:
            return '{}, {}'.format(self.school_name, self.school_class)
        else:
            return self.school_name or self.school_class or ''


class BillingInfo(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE,
                             related_name='leprikon_billing_info', verbose_name=_('user'))
    name = models.CharField(_('name'), max_length=150)
    street = models.CharField(_('street'), max_length=150, blank=True, null=True)
    city = models.CharField(_('city'), max_length=150, blank=True, null=True)
    postal_code = PostalCodeField(_('postal code'), blank=True, null=True)
    company_num = models.CharField(_('company number'), max_length=8, blank=True, null=True)
    vat_number = models.CharField(_('VAT number'), max_length=10, blank=True, null=True)
    contact_person = models.CharField(_('contact person'), max_length=60, blank=True, null=True)
    phone = models.CharField(_('phone'), max_length=30, blank=True, default='')
    email = EmailField(_('email address'), blank=True, default='')
    employee = models.CharField(_('employee ID'), max_length=150, blank=True, default='')

    class Meta:
        app_label = 'leprikon'
        ordering = ('name',)
        verbose_name = _('billing information')
        verbose_name_plural = _('billing information')

    def __str__(self):
        return self.name

    @cached_property
    def address(self):
        return ', '.join(filter(bool, (self.street, self.city, self.postal_code)))
    address.short_description = _('address')


class LeaderPlugin(CMSPlugin):
    leader = models.ForeignKey(Leader, on_delete=models.CASCADE, related_name='+', verbose_name=_('leader'))
    template = models.CharField(
        _('template'), max_length=100,
        choices=settings.LEPRIKON_LEADER_TEMPLATES,
        default=settings.LEPRIKON_LEADER_TEMPLATES[0][0],
        help_text=_('The template used to render plugin.'),
    )

    class Meta:
        app_label = 'leprikon'


class LeaderListPlugin(CMSPlugin):
    school_year = models.ForeignKey(SchoolYear, blank=True, null=True, on_delete=models.CASCADE,
                                    related_name='+', verbose_name=_('school year'))
    subject = models.ForeignKey('leprikon.Subject', blank=True, null=True, on_delete=models.CASCADE,
                                related_name='+', verbose_name=_('subject'))
    template = models.CharField(
        _('template'), max_length=100,
        choices=settings.LEPRIKON_LEADERLIST_TEMPLATES,
        default=settings.LEPRIKON_LEADERLIST_TEMPLATES[0][0],
        help_text=_('The template used to render plugin.'),
    )

    class Meta:
        app_label = 'leprikon'

    def clean(self):
        if self.school_year and self.subject and self.subject.school_year != self.school_year:
            raise ValidationError({
                'school_year': [_('Selected subject is not in the selected school year.')],
                'subject': [_('Selected subject is not in the selected school year.')],
            })

    def render(self, context):
        if self.subject:
            leaders = self.subject.leaders.all()
        else:
            school_year = (self.school_year or getattr(context.get('request'), 'school_year') or
                           SchoolYear.objects.get_current())
            leaders = school_year.leaders.all()
        context.update({
            'leaders': leaders,
        })
        return context


class FilteredLeaderListPlugin(CMSPlugin):
    school_year = models.ForeignKey(SchoolYear, blank=True, null=True, on_delete=models.CASCADE,
                                    related_name='+', verbose_name=_('school year'))

    class Meta:
        app_label = 'leprikon'

    def render(self, context):
        school_year = (self.school_year or getattr(context.get('request'), 'school_year') or
                       SchoolYear.objects.get_current())
        form = LeaderFilterForm(
            school_year=school_year,
            data=context['request'].GET,
        )
        context.update({
            'school_year': school_year,
            'form': form,
            'leaders': form.get_queryset(),
        })
        return context



from django.db import models
from django.utils.translation import ugettext_lazy as _


class Place(models.Model):
    name = models.CharField(_('name'), max_length=50)
    place = models.CharField(_('place'), max_length=50, blank=True, default='')

    class Meta:
        app_label = 'leprikon'
        ordering = ('place',)
        verbose_name = _('place')
        verbose_name_plural = _('places')

    def __str__(self):
        if self.place:
            return '{name}, {place}'.format(name=self.name, place=self.place)
        else:
            return '{name}'.format(name=self.name)



from django.db import models
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from localflavor.generic.models import BICField, IBANField

from .fields import EmailField, PostalCodeField
from .utils import BankAccount


class Organization(models.Model):
    name = models.CharField(_('organization name'), max_length=150)
    street = models.CharField(_('street'), max_length=150, blank=True, null=True)
    city = models.CharField(_('city'), max_length=150, blank=True, null=True)
    postal_code = PostalCodeField(_('postal code'), blank=True, null=True)
    email = EmailField(_('email address'), blank=True, null=True)
    phone = models.CharField(_('phone'), max_length=30, blank=True, null=True)
    company_num = models.CharField(_('company number'), max_length=8, blank=True, null=True)
    vat_number = models.CharField(_('VAT number'), max_length=10, blank=True, null=True)
    iban = IBANField(_('IBAN'), blank=True, null=True)
    bic = BICField(_('BIC (SWIFT)'), blank=True, null=True)

    class Meta:
        app_label = 'leprikon'
        verbose_name = _('organization')
        verbose_name_plural = _('organizations')

    def __str__(self):
        return f'{self.name} ({self.bank_account})'

    @cached_property
    def address(self):
        return ', '.join(filter(bool, (self.street, self.city, self.postal_code)))
    address.short_description = _('address')

    @cached_property
    def bank_account(self):
        return self.iban and BankAccount(self.iban)
    bank_account.short_description = _('bank account')



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _

from . import bankreaders  # noqa


class LeprikonConfig(AppConfig):
    name = 'leprikon'
    verbose_name = _('Leprikon')

    def ready(self):

        # ensure that current LeprikonSite exists
        from .models.leprikonsite import LeprikonSite
        try:
            LeprikonSite.objects.get_current()
        except Exception:
            pass

        # create leprikon page on first run
        from cms.api import create_page
        from cms.constants import TEMPLATE_INHERITANCE_MAGIC
        from menus.menu_pool import menu_pool
        from .conf import settings
        try:
            create_page(
                title='Leprikón',
                template=TEMPLATE_INHERITANCE_MAGIC,
                language=settings.LANGUAGE_CODE,
                slug='leprikon',
                apphook='LeprikonApp',
                apphook_namespace='leprikon',
                reverse_id='leprikon',
                in_navigation=True,
                navigation_extenders='LeprikonMenu',
                published=True,
            ).set_as_homepage()
            menu_pool.clear()
        except Exception:
            pass



from time import time

from django.contrib.sites.models import Site
from django.db import models
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from django_pays.models import Gateway
from djangocms_text_ckeditor.fields import HTMLField

from ..conf import settings
from .account import AccountClosure
from .agreements import Agreement
from .organizations import Organization
from .printsetup import PrintSetup


class LeprikonSiteManager(models.Manager):
    _cached_site = None
    _cache_timestamp = None

    def get_current(self):
        now = time()
        if not self._cached_site or now - self._cache_timestamp > settings.LEPRIKON_SITE_CACHE_MAX_AGE:
            lookup_kwargs = {}
            if getattr(settings, 'SITE_ID', ''):
                lookup_kwargs['pk'] = settings.SITE_ID
            self._cached_site = self.get_or_create(**lookup_kwargs)[0]
            self._cache_timestamp = now
        return self._cached_site

    def get_queryset(self):
        return super(LeprikonSiteManager, self).get_queryset().select_related(
            'bill_print_setup',
            'reg_print_setup',
        )


class LeprikonSite(Site):
    organization = models.ForeignKey(Organization, blank=True, null=True, on_delete=models.SET_NULL,
                                     related_name='+', verbose_name=_('default organization'))
    bill_print_setup = models.ForeignKey(PrintSetup, blank=True, null=True, on_delete=models.SET_NULL,
                                         related_name='+', verbose_name=_('default bill print setup'))
    reg_print_setup = models.ForeignKey(PrintSetup, blank=True, null=True, on_delete=models.SET_NULL,
                                        related_name='+', verbose_name=_('default registration print setup'))
    user_agreement = HTMLField(_('user agreement'), blank=True, default='')
    user_agreement_changed = models.DateTimeField(_('last time user agreement changed'), blank=True, null=True)
    registration_agreements = models.ManyToManyField(
        Agreement, blank=True, related_name='+', verbose_name=_('default registration agreements'),
        help_text=_('Add legal agreements for the registration form.'),
    )
    payment_gateway = models.ForeignKey(Gateway, blank=True, null=True, on_delete=models.SET_NULL,
                                        related_name='+', verbose_name=_('Payment Gateway'))

    objects = LeprikonSiteManager()

    class Meta:
        app_label = 'leprikon'
        verbose_name = _('leprikon site')
        verbose_name_plural = _('leprikon sites')

    @cached_property
    def max_closure_date(self):
        return AccountClosure.objects.max_closure_date()

    @cached_property
    def url(self):
        return settings.LEPRIKON_URL or f'https://{self.domain}'



from io import BytesIO

from django.conf.urls import url as urls_url
from django.http import HttpResponse
from django.urls import reverse
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _
from PyPDF2 import PdfFileReader, PdfFileWriter


class PdfExportAdminMixin(object):
    actions = ('export_pdf',)

    def export_pdf(self, request, queryset):
        # create PDF
        writer = PdfFileWriter()
        for obj in queryset.iterator():
            pdf_data = BytesIO()
            obj.write_pdf(pdf_data)
            pdf_data.seek(0)
            pdf = PdfFileReader(pdf_data)
            for i in range(pdf.getNumPages()):
                writer.addPage(pdf.getPage(i))

        # create PDF response object
        response = HttpResponse(content_type='application/pdf')
        response['Content-Disposition'] = 'attachment; filename="{}.pdf"'.format(
            slugify(self.model._meta.verbose_name_plural),
        )

        # write PDF to response
        writer.write(response)

        return response
    export_pdf.short_description = _('Export selected items in single PDF.')

    def get_urls(self):
        urls = super(PdfExportAdminMixin, self).get_urls()
        return [urls_url(
            r'(?P<obj_id>\d+).pdf$',
            self.admin_site.admin_view(self.pdf),
            name='{}_{}_pdf'.format(self.model._meta.app_label, self.model._meta.model_name),
        )] + urls

    def pdf(self, request, obj_id):
        obj = self.get_object(request, obj_id)

        # create PDF response object
        response = HttpResponse(content_type='application/pdf')
        response['Content-Disposition'] = 'attachment; filename="{}"'.format(obj.pdf_filename)

        # write PDF to response
        return obj.write_pdf(response)

    def download_tag(self, obj):
        return '<a href="{}">PDF</a>'.format(reverse(
            'admin:{}_{}_pdf'.format(self.model._meta.app_label, self.model._meta.model_name),
            args=(obj.id,),
        ))
    download_tag.short_description = _('download')
    download_tag.allow_tags = True



from collections import namedtuple
from datetime import date, datetime, timedelta

from cms.models import CMSPlugin
from django.db import models
from django.utils.formats import date_format, time_format
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from ..conf import settings
from .agegroup import AgeGroup
from .department import Department
from .roles import Leader
from .schoolyear import SchoolYear
from .subjects import (
    Subject, SubjectDiscount, SubjectGroup, SubjectRegistration, SubjectType,
)
from .utils import PaymentStatus, change_year


class Event(Subject):
    start_date = models.DateField(_('start date'))
    end_date = models.DateField(_('end date'))
    start_time = models.TimeField(_('start time'), blank=True, null=True)
    end_time = models.TimeField(_('end time'), blank=True, null=True)
    due_from = models.DateField(_('due from'))
    due_date = models.DateField(_('due date'))

    class Meta:
        app_label = 'leprikon'
        ordering = ('start_date', 'start_time')
        verbose_name = _('event')
        verbose_name_plural = _('events')

    @property
    def inactive_registrations(self):
        return self.registrations.filter(canceled__isnull=False)

    def get_times_list(self):
        return '{start}{separator}{end}'.format(
            start=(date_format(datetime.combine(self.start_date, self.start_time), 'SHORT_DATETIME_FORMAT')
                   if self.start_time else date_format(self.start_date, 'SHORT_DATE_FORMAT')),
            separator=' - ' if self.start_date != self.end_date or self.end_time is not None else '',
            end=((time_format(self.end_time, 'TIME_FORMAT') if self.end_time else '')
                 if self.start_date == self.end_date
                 else (date_format(datetime.combine(self.end_date, self.end_time), 'SHORT_DATETIME_FORMAT')
                       if self.end_time else date_format(self.end_date, 'SHORT_DATE_FORMAT'))),
        )
    get_times_list.short_description = _('times')

    Time = namedtuple('Time', ('date', 'start', 'end'))

    def get_next_time(self, now=None):
        if not now:
            return self.Time(
                date=self.start_date,
                start=self.start_time,
                end=(self.end_time if self.end_date == self.start_date else None),
            )
        else:
            next_date = (now.date() if isinstance(now, datetime) else now) + timedelta(1)
            return self.Time(
                date=next_date,
                start=None,
                end=(self.end_time if self.end_date == next_date else None),
            )

    def copy_to_school_year(old, school_year):
        new = Event.objects.get(id=old.id)
        new.id, new.pk = None, None
        new.school_year = school_year
        new.public = False
        new.evaluation = ''
        new.note = ''
        year_delta = school_year.year - old.school_year.year
        new.reg_from = new.reg_from and change_year(new.reg_from, year_delta)
        new.reg_to = new.reg_to and change_year(new.reg_to, year_delta)
        new.due_from = change_year(new.due_from, year_delta)
        new.due_date = change_year(new.due_date, year_delta)
        new.start_date = change_year(new.start_date, year_delta)
        new.end_date = change_year(new.end_date, year_delta)
        new.save()
        new.groups.set(old.groups.all())
        new.age_groups.set(old.age_groups.all())
        new.leaders.set(old.leaders.all())
        new.questions.set(old.questions.all())
        new.attachments.set(old.attachments.all())
        return new


class EventRegistration(SubjectRegistration):
    subject_type = SubjectType.EVENT

    class Meta:
        app_label = 'leprikon'
        verbose_name = _('event registration')
        verbose_name_plural = _('event registrations')

    def get_payment_status(self, d=None):
        if d is None:
            d = date.today()
        explanation = ',\n'.join(
            discount.explanation.strip()
            for discount in self.all_discounts
            if discount.accounted.date() <= d and discount.explanation.strip()
        )
        return PaymentStatus(
            price=self.price,
            discount=self.get_discounted(d),
            explanation=explanation,
            paid=self.get_paid(d),
            current_date=d,
            due_from=self.subject.event.due_from,
            due_date=self.subject.event.due_date,
        )

    @cached_property
    def payment_status(self):
        return self.get_payment_status()

    @cached_property
    def payment_statuses(self):
        return [self.payment_status]


class EventDiscount(SubjectDiscount):
    registration = models.ForeignKey(EventRegistration, on_delete=models.CASCADE,
                                     related_name='discounts', verbose_name=_('registration'))

    class Meta:
        app_label = 'leprikon'
        verbose_name = _('event discount')
        verbose_name_plural = _('event discounts')
        ordering = ('accounted',)


class EventPlugin(CMSPlugin):
    event = models.ForeignKey(Event, on_delete=models.CASCADE, related_name='+', verbose_name=_('event'))
    template = models.CharField(_('template'), max_length=100,
                                choices=settings.LEPRIKON_EVENT_TEMPLATES,
                                default=settings.LEPRIKON_EVENT_TEMPLATES[0][0],
                                help_text=_('The template used to render plugin.'))

    class Meta:
        app_label = 'leprikon'


class EventListPlugin(CMSPlugin):
    school_year = models.ForeignKey(SchoolYear, blank=True, null=True, on_delete=models.CASCADE,
                                    related_name='+', verbose_name=_('school year'))
    departments = models.ManyToManyField(Department, blank=True, related_name='+', verbose_name=_('departments'),
                                         help_text=_('Keep empty to skip searching by departments.'))
    event_types = models.ManyToManyField(SubjectType, blank=True,
                                         limit_choices_to={'subject_type': SubjectType.EVENT},
                                         related_name='+', verbose_name=_('event types'),
                                         help_text=_('Keep empty to skip searching by event types.'))
    age_groups = models.ManyToManyField(AgeGroup, blank=True, related_name='+', verbose_name=_('age groups'),
                                        help_text=_('Keep empty to skip searching by age groups.'))
    groups = models.ManyToManyField(SubjectGroup, blank=True, related_name='+', verbose_name=_('event groups'),
                                    help_text=_('Keep empty to skip searching by groups.'))
    leaders = models.ManyToManyField(Leader, verbose_name=_('leaders'), blank=True, related_name='+',
                                     help_text=_('Keep empty to skip searching by leaders.'))
    template = models.CharField(_('template'), max_length=100,
                                choices=settings.LEPRIKON_EVENTLIST_TEMPLATES,
                                default=settings.LEPRIKON_EVENTLIST_TEMPLATES[0][0],
                                help_text=_('The template used to render plugin.'))

    class Meta:
        app_label = 'leprikon'

    def copy_relations(self, oldinstance):
        self.departments = oldinstance.departments.all()
        self.event_types = oldinstance.event_types.all()
        self.groups = oldinstance.groups.all()
        self.age_groups = oldinstance.age_groups.all()
        self.leaders = oldinstance.leaders.all()

    @cached_property
    def all_departments(self):
        return list(self.departments.all())

    @cached_property
    def all_event_types(self):
        return list(self.event_types.all())

    @cached_property
    def all_age_groups(self):
        return list(self.age_groups.all())

    @cached_property
    def all_groups(self):
        return list(self.groups.all())

    @cached_property
    def all_leaders(self):
        return list(self.leaders.all())

    Group = namedtuple('Group', ('group', 'objects'))

    def render(self, context):
        school_year = (self.school_year or getattr(context.get('request'), 'school_year') or
                       SchoolYear.objects.get_current())
        events = Event.objects.filter(school_year=school_year, public=True).distinct()

        if self.all_departments:
            events = events.filter(department__in=self.all_departments)
        if self.all_event_types:
            events = events.filter(subject_type__in=self.all_event_types)
        if self.all_age_groups:
            events = events.filter(age_groups__in=self.all_age_groups)
        if self.all_leaders:
            events = events.filter(leaders__in=self.all_leaders)
        if self.all_groups:
            events = events.filter(groups__in=self.all_groups)
            groups = self.all_groups
        elif self.all_event_types:
            groups = SubjectGroup.objects.filter(subject_types__in=self.all_event_types)
        else:
            groups = SubjectGroup.objects.all()

        context.update({
            'school_year': school_year,
            'events': events,
            'groups': (
                self.Group(group=group, objects=events.filter(groups=group))
                for group in groups
            ),
        })
        return context


class FilteredEventListPlugin(CMSPlugin):
    school_year = models.ForeignKey(SchoolYear, blank=True, null=True, on_delete=models.CASCADE,
                                    related_name='+', verbose_name=_('school year'))
    event_types = models.ManyToManyField(SubjectType, limit_choices_to={'subject_type': SubjectType.EVENT},
                                         related_name='+', verbose_name=_('event types'))

    class Meta:
        app_label = 'leprikon'

    def copy_relations(self, oldinstance):
        self.event_types = oldinstance.event_types.all()

    @cached_property
    def all_event_types(self):
        return list(self.event_types.all())

    def render(self, context):
        school_year = (self.school_year or getattr(context.get('request'), 'school_year') or
                       SchoolYear.objects.get_current())

        from ..forms.subjects import SubjectFilterForm
        form = SubjectFilterForm(
            subject_type_type=SubjectType.EVENT,
            subject_types=self.all_event_types,
            school_year=school_year,
            is_staff=context['request'].user.is_staff,
            data=context['request'].GET,
        )
        context.update({
            'school_year': school_year,
            'form': form,
            'events': form.get_queryset(),
        })
        return context



from django.db import models
from django.urls import reverse
from django.utils import timezone
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from .leprikonsite import LeprikonSite
from .schoolyear import SchoolYear
from .subjects import Subject, SubjectType


class RegistrationLink(models.Model):
    school_year = models.ForeignKey(SchoolYear, editable=False, on_delete=models.CASCADE,
                                    related_name='registration_links', verbose_name=_('school year'))
    subject_type = models.ForeignKey(SubjectType, on_delete=models.CASCADE,
                                     related_name='registration_links', verbose_name=_('subject type'))
    slug = models.SlugField(editable=False, max_length=64)
    name = models.CharField(_('name'), max_length=50)
    reg_from = models.DateTimeField(_('registration active from'))
    reg_to = models.DateTimeField(_('registration active to'))
    subjects = models.ManyToManyField(Subject, blank=False, related_name='registration_links',
                                      verbose_name=_('subjects'))

    class Meta:
        app_label = 'leprikon'
        ordering = ('reg_from', 'reg_to')
        verbose_name = _('registration link')
        verbose_name_plural = _('registration links')

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('leprikon:registration_link', args=(self.slug,))

    @cached_property
    def link(self):
        return LeprikonSite.objects.get_current().url + self.get_absolute_url()

    @property
    def registration_allowed(self):
        now = timezone.now()
        return (self.reg_from <= now) and (self.reg_to > now)
